// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-

const Clutter = imports.gi.Clutter;
const Lang = imports.lang;
const Mainloop = imports.mainloop;
const Shell = imports.gi.Shell;
const Gvc = imports.gi.Gvc;
const Signals = imports.signals;
const St = imports.gi.St;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const Util = imports.misc.util;


let advancedAudioMenu;
const SHELL_VOLUME = "org.gnome.Shell";
const ICON_SIZE = 32;

function AppIndicator(){
    this._init.apply(this,arguments);
}

AppIndicator.prototype = {
    
    __proto__: PopupMenu.PopupBaseMenuItem.prototype,

    _init: function(sink,id,container){
        PopupMenu.PopupBaseMenuItem.prototype._init.call(this);
        this.container =container;
        this.sink = sink;
        this.appSystem = Shell.AppSystem.get_default();                
        let app = this.appSystem.lookup_app(id);
        if(app ==null) app = this.appSystem.lookup_app(this.sink.get_icon_name()+".desktop");
        let value = this.sink.volume/ this.container.volumeMax;
        this._slider = new PopupMenu.PopupSliderMenuItem(value);
        this._slider.connect('value-changed', Lang.bind(this, this._sliderChanged));
        this._slider.connect('drag-end', Lang.bind(this, this._notifyVolumeChange));
        this._box  = new St.BoxLayout({ vertical:true});
        this._icons = new St.BoxLayout();
        this._label = new St.Label({ text: id,style_class: 'appvolume-label' });
        this._icon =app !=null ? app.create_icon_texture(ICON_SIZE) : new St.Icon({icon_name: "audio-x-generic", icon_size: 2*ICON_SIZE, icon_type: St.IconType.SYMBOLIC});        
        this._switch =  new PopupMenu.PopupSwitchMenuItem("",!this.sink.is_muted);         
        this._switch.connect('toggled', Lang.bind(this, this._switchToggled));        
        this._icons.add_actor(this._icon);
        this._icons.add_actor(this._label);        
        this._icons.add_actor(this._switch.actor);         
        this._box.add_actor(this._icons);
        this._box.add_actor(this._slider.actor);        
        this.addActor(this._box,{ align: St.Align.MIDDLE });           
    },
    _notifyVolumeChange: function() {
       
    },
    _switchToggled : function(_switch) {
        this.sink.change_is_muted(!(_switch.state));
        this.sink.push_volume();
    },
    _sliderChanged: function(slider, value, property) {
        if (this.sink == null) {
            log ('Volume slider changed for %s, but %s does not exist'.format(property, property));
            return;
        }
        let volume = value * this.container.volumeMax;
        let prev_muted = this.sink.is_muted;
        if (volume < 1) {
            this.sink.volume = 0;
            if (!prev_muted){
                this.sink.change_is_muted(true);
                this._switch.setToggleState(false);
            }
        } else {
            this.sink.volume = volume;
            if (prev_muted){
                this.sink.change_is_muted(false);
                this._switch.setToggleState(true);
            }
        }
        this.sink.push_volume();
    }
};

function AdvancedAudioMenu(){
    this._init();
}

AdvancedAudioMenu.prototype = {
    _init : function() {
        this.indicatorsArray = {};
        this.indicator = Main.panel._statusArea["volume"];
        this.volumeMax = this.indicator._control.get_vol_max_norm();
        this.conn_added = this.indicator._control.connect('stream-added',Lang.bind(this,this.showApplicationVolumes));
        this.conn_removed = this.indicator._control.connect('stream-removed', Lang.bind(this,this.showApplicationVolumes));
        this.showApplicationVolumes();
    },
    
    showApplicationVolumes :function () {
        let showInput = false;
        let recordingApps = this.indicator._control.get_sink_inputs()
        this.clearApplicationVolumes();
        if (recordingApps) {
            for (let i = 0; i < recordingApps.length; i++) {
                let outputStream = recordingApps[i];
                let id = outputStream.get_application_id()!=null ? outputStream.get_application_id() : outputStream.get_name();
                if(id && this.indicatorsArray[id]==null && id != SHELL_VOLUME){
                    this.indicatorsArray[id] = new AppIndicator(outputStream,id,this);
                    this.indicator.menu.addMenuItem(this.indicatorsArray[id]); 
                    if(i != recordingApps.length-1){
                        let nextId = recordingApps[i+1].get_application_id()!=null ? recordingApps[i+1].get_application_id() : recordingApps[i+1].get_name();  
                        if(nextId != SHELL_VOLUME){ 
                            this.indicatorsArray[id+i] = new PopupMenu.PopupSeparatorMenuItem();
                            this.indicator.menu.addMenuItem(this.indicatorsArray[id+i]);
                        }
                    }
                }
            }
        }
    },
    
    clearApplicationVolumes : function(){
        for (let p in this.indicatorsArray){
            this.indicatorsArray[p].destroy();
        }
        this.indicatorsArray = {};
    },
    
    destroy : function(){
        this.indicator._control.disconnect(this.conn_added);
        this.indicator._control.disconnect(this.conn_removed);
        this.clearApplicationVolumes();
    }
};


function init() {
}

function enable() {
    advancedAudioMenu =new AdvancedAudioMenu();
}
function disable() {
    advancedAudioMenu.destroy();
}
